/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIMediaManager.idl
 */

#ifndef __gen_nsIMediaManager_h__
#define __gen_nsIMediaManager_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsISupportsArray; /* forward declaration */

class nsIDOMWindow; /* forward declaration */

#define NS_MEDIAMANAGERSERVICE_CID {0xabc622ea, 0x9655, 0x4123, {0x80, 0xd9, 0x22, 0x62, 0x1b, 0xdd, 0x54, 0x65}}
#define MEDIAMANAGERSERVICE_CONTRACTID "@mozilla.org/mediaManagerService;1"

/* starting interface:    nsIMediaManagerService */
#define NS_IMEDIAMANAGERSERVICE_IID_STR "2ab0e6f7-9a5b-4b9a-901d-145531f47a6b"

#define NS_IMEDIAMANAGERSERVICE_IID \
  {0x2ab0e6f7, 0x9a5b, 0x4b9a, \
    { 0x90, 0x1d, 0x14, 0x55, 0x31, 0xf4, 0x7a, 0x6b }}

class NS_NO_VTABLE nsIMediaManagerService : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IMEDIAMANAGERSERVICE_IID)

  /* readonly attribute nsISupportsArray activeMediaCaptureWindows; */
  NS_IMETHOD GetActiveMediaCaptureWindows(nsISupportsArray * *aActiveMediaCaptureWindows) = 0;

  /* void mediaCaptureWindowState (in nsIDOMWindow aWindow, out boolean aVideo, out boolean aAudio, [optional] out boolean aScreenShare, [optional] out boolean aWindowShare, [optional] out boolean aAppShare); */
  NS_IMETHOD MediaCaptureWindowState(nsIDOMWindow *aWindow, bool *aVideo, bool *aAudio, bool *aScreenShare, bool *aWindowShare, bool *aAppShare) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIMediaManagerService, NS_IMEDIAMANAGERSERVICE_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIMEDIAMANAGERSERVICE \
  NS_IMETHOD GetActiveMediaCaptureWindows(nsISupportsArray * *aActiveMediaCaptureWindows) MOZ_OVERRIDE; \
  NS_IMETHOD MediaCaptureWindowState(nsIDOMWindow *aWindow, bool *aVideo, bool *aAudio, bool *aScreenShare, bool *aWindowShare, bool *aAppShare) MOZ_OVERRIDE; 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIMEDIAMANAGERSERVICE(_to) \
  NS_IMETHOD GetActiveMediaCaptureWindows(nsISupportsArray * *aActiveMediaCaptureWindows) MOZ_OVERRIDE { return _to GetActiveMediaCaptureWindows(aActiveMediaCaptureWindows); } \
  NS_IMETHOD MediaCaptureWindowState(nsIDOMWindow *aWindow, bool *aVideo, bool *aAudio, bool *aScreenShare, bool *aWindowShare, bool *aAppShare) MOZ_OVERRIDE { return _to MediaCaptureWindowState(aWindow, aVideo, aAudio, aScreenShare, aWindowShare, aAppShare); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIMEDIAMANAGERSERVICE(_to) \
  NS_IMETHOD GetActiveMediaCaptureWindows(nsISupportsArray * *aActiveMediaCaptureWindows) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->GetActiveMediaCaptureWindows(aActiveMediaCaptureWindows); } \
  NS_IMETHOD MediaCaptureWindowState(nsIDOMWindow *aWindow, bool *aVideo, bool *aAudio, bool *aScreenShare, bool *aWindowShare, bool *aAppShare) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->MediaCaptureWindowState(aWindow, aVideo, aAudio, aScreenShare, aWindowShare, aAppShare); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsMediaManagerService : public nsIMediaManagerService
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIMEDIAMANAGERSERVICE

  nsMediaManagerService();

private:
  ~nsMediaManagerService();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsMediaManagerService, nsIMediaManagerService)

nsMediaManagerService::nsMediaManagerService()
{
  /* member initializers and constructor code */
}

nsMediaManagerService::~nsMediaManagerService()
{
  /* destructor code */
}

/* readonly attribute nsISupportsArray activeMediaCaptureWindows; */
NS_IMETHODIMP nsMediaManagerService::GetActiveMediaCaptureWindows(nsISupportsArray * *aActiveMediaCaptureWindows)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void mediaCaptureWindowState (in nsIDOMWindow aWindow, out boolean aVideo, out boolean aAudio, [optional] out boolean aScreenShare, [optional] out boolean aWindowShare, [optional] out boolean aAppShare); */
NS_IMETHODIMP nsMediaManagerService::MediaCaptureWindowState(nsIDOMWindow *aWindow, bool *aVideo, bool *aAudio, bool *aScreenShare, bool *aWindowShare, bool *aAppShare)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIMediaManager_h__ */
