/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/**
 * This is the interface to the Popup Window Manager: an object which
 * maintains popup window permissions by website.
 */

#include "nsISupports.idl"

interface nsIPrincipal;

[scriptable, uuid(66386aa9-2088-4bae-82c7-9f58bc02be64)]
interface nsIPopupWindowManager : nsISupports {

  /**
   * These values are returned by the testPermission method
   */
  const uint32_t ALLOW_POPUP = 1;
  const uint32_t DENY_POPUP = 2;
  const uint32_t ALLOW_POPUP_WITH_PREJUDICE = 3;

  /**
   * Test whether a website has permission to show a popup window.
   * @param   principal is the principal to be tested
   * @return  one of the enumerated permission actions defined above
   */
  uint32_t testPermission(in nsIPrincipal principal);
};

%{ C++
#define NS_POPUPWINDOWMANAGER_CONTRACTID "@mozilla.org/PopupWindowManager;1"
%}
